// -*- C++ -*-
/*!
 * @file BufferBase.h
 * @brief Buffer abstract class
 * @date $Date: 2007/09/21 09:18:12 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Noriaki Ando
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: BufferBase.h,v 1.2.4.2 2007/09/21 09:18:12 n-ando Exp $
 *
 */

/*
 * $Log: BufferBase.h,v $
 * Revision 1.2.4.2  2007/09/21 09:18:12  n-ando
 * Documentation was modified.
 *
 * Revision 1.2  2006/12/02 18:24:15  n-ando
 * BufferBase's interfaces were changed.
 *
 * Revision 1.1  2006/11/27 09:44:34  n-ando
 * The first commitment.
 *
 */

#ifndef BufferBase_h
#define BufferBase_h

namespace RTC
{
  /*!
   * @if jp
   * @class BufferBase
   * @brief BufferBase ۃNX
   * 
   * X̃obt@̂߂̒ۃC^[tF[XNXB
   * ۃobt@NX́Aȉ̏z֐̎񋟂Ȃ΂ȂȂB
   *
   * publicC^[tF[XƂĈȉ̂̂񋟂B
   * - write(): obt@ɏ
   * - read(): obt@ǂݏo
   * - length(): obt@Ԃ
   * - isFull(): obt@tł
   * - isEmpty(): obt@ł
   *
   * protectedC^[tF[XƂĈȉ̂̂񋟂B
   * - put(): obt@Ƀf[^
   * - get(): obt@f[^ǂݏo
   *
   *
   *
   * @param DataType obt@Ɋi[f[^^
   *
   * @else
   * @class BufferBase
   * @brief BufferBase abstract class
   *
   * This is the abstract interface class for various Buffer.
   *
   * @param DataType Data type to be stored to the buffer.
   *
   * @endif
   */
  template <class DataType>
  class BufferBase
  {
  public:
    /*!
     * @if jp
     *
     * @brief zfXgN^
     * 
     * @else
     *
     * @brief virtual destractor
     *
     * @endif
     */
    virtual ~BufferBase(){};

    /*!
     * @if jp
     *
     * @brief obt@̒擾
     * 
     * @else
     *
     * @brief Get the buffer length
     *
     * @endif
     */
    virtual long int length() const = 0;

    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual bool write(const DataType& value) = 0;

    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual bool read(DataType& value) = 0;

    /*!
     * @if jp
     *
     * @brief obt@fullł
     * 
     * @else
     *
     * @brief True if the buffer is full, else false.
     *
     * @endif
     */
    virtual bool isFull() const = 0;

    /*!
     * @if jp
     *
     * @brief obt@emptył
     * 
     * @else
     *
     * @brief True if the buffer is empty, else false.
     *
     * @endif
     */
    virtual bool isEmpty() const = 0;

  protected:
    /*!
     * @if jp
     *
     * @brief obt@Ƀf[^
     * 
     * @else
     *
     * @brief Write data into the buffer
     *
     * @endif
     */
    virtual void put(const DataType& data) = 0;

    /*!
     * @if jp
     *
     * @brief obt@f[^擾
     * 
     * @else
     *
     * @brief Get data from the buffer
     *
     * @endif
     */
    virtual const DataType& get() = 0;

    /*!
     * @if jp
     *
     * @brief ɏރobt@̎QƂ擾
     * 
     * @else
     *
     * @brief Get the buffer's reference to be written the next
     *
     * @endif
     */
    virtual DataType& getRef() = 0;

  };




  template <class DataType>
  class NullBuffer
    : public BufferBase<DataType>
  {
  public:
    NullBuffer(long int size = 1)
      : m_length(1)
    {
    }

    virtual ~NullBuffer()
    {
    }

    virtual long int length() const
    {
      return 1;
    }

    virtual bool write(const DataType& value)
    {
      m_data = value;
      return true;
    }

    virtual bool read(DataType& value)
    {
      value = m_data;
      return true;
    }

    virtual bool isFull() const
    {
      return false;
    }

    virtual bool isEmpty() const
    {
      return false;
    }
    
  protected:
    virtual void put(const DataType& data)
    {
      m_data = data;
    }

    virtual const DataType& get()
    {
      return m_data;
    }

    virtual DataType& getRef()
    {
      return m_data;
    }

  private:
    DataType m_data;
    long int m_length;
  };


}; // namespace RTC
#endif // BufferBase_h
