// -*- C++ -*-
/*!
 * @file PortAdmin.h
 * @brief RTC's Port administration class
 * @date $Date: 2007/09/20 11:25:11 $
 * @author Noriaki Ando <n-ando@aist.go.jp>
 *
 * Copyright (C) 2006
 *     Task-intelligence Research Group,
 *     Intelligent Systems Research Institute,
 *     National Institute of
 *         Advanced Industrial Science and Technology (AIST), Japan
 *     All rights reserved.
 *
 * $Id: PortAdmin.h,v 1.6.4.1 2007/09/20 11:25:11 n-ando Exp $
 *
 */

/*
 * $Log: PortAdmin.h,v $
 * Revision 1.6.4.1  2007/09/20 11:25:11  n-ando
 * A function getPortProfileList() was added to get PortProfileList locally.
 *
 * Revision 1.6  2007/01/21 10:28:09  n-ando
 * Some trivial fixes.
 *
 * Revision 1.5  2007/01/09 15:13:11  n-ando
 * Now Port does not have getName().
 * The name of Port is obtained from getProfile().
 *
 * Revision 1.4  2006/12/02 18:49:43  n-ando
 * A trivial fix.
 *
 * Revision 1.3  2006/11/06 01:19:04  n-ando
 * CORBA sequence manipulation has been rewritten by using CORBA_SeqUtil.
 *
 * Revision 1.2  2006/10/17 19:16:40  n-ando
 * registerPort() was modified to store Port's object reference in PortProfile.
 *
 * Revision 1.1  2006/10/17 10:22:01  n-ando
 * The first commitment.
 *
 */

#ifndef PortAdmin_h
#define PortAdmin_h

#include <rtm/idl/RTCSkel.h>
#include <rtm/PortBase.h>
#include <rtm/ObjectManager.h>
// 68608 10 17 23:51 PortAdmin.o
// 54272 10 29 00:53 PortAdmin.o
// 54132 10 29 00:54 PortAdmin.o
namespace RTC
{


  class PortAdmin
  {
  public:
    PortAdmin(CORBA::ORB_ptr, PortableServer::POA_ptr poa);
    virtual ~PortAdmin(){};

    /*!
     * @if jp
     *
     * @brief PortList ̎擾
     *
     * registerPort() ɂo^ꂽ Port  PortList ւ̃|C^ԂB
     *
     * @return PortList* PortList ւ̃|C^
     *
     * @else
     *
     * @brief Get PortList
     *
     * This operation returns the pointer to the PortList of Ports regsitered
     * by registerPort().
     *
     * @return PortList+ The pointer points PortList
     *
     * @endif
     */
    PortList* getPortList() const;

    PortProfileList getPortProfileList() const;
    /*!
     * @if jp
     *
     * @brief Port ̃IuWFNgQƂ̎擾
     *
     * port_name Ŏw肵 Port ̃IuWFNgQƂԂB
     * port_name Ŏw肷 Port ͂炩 registerPort() œo^Ă
     * Ȃ΂ȂȂB
     *
     * @param port_name QƂԂPort̖O
     * @return Port_ptr Port̃IuWFNgQ
     *
     * @else
     *
     * @brief Get PortList
     *
     * This operation returns the pointer to the PortList of Ports regsitered
     * by registerPort().
     *
     * @param port_name The name of Port to be returned the reference.
     * @return Port_ptr Port's object reference.
     *
     * @endif
     */
    Port_ptr getPortRef(const char* port_name) const;


    /*!
     * @if jp
     *
     * @brief Port ̃T[og̃|C^̎擾
     *
     * port_name Ŏw肵 Port ̃T[og̃|C^ԂB
     * port_name Ŏw肷 Port ͂炩 registerPort() œo^Ă
     * Ȃ΂ȂȂB
     *
     * @param port_name QƂԂPort̖O
     * @return PortBase* PortT[ogNX̃|C^
     *
     * @else
     *
     * @brief Getpointer to the Port's servant
     *
     * This operation returns the pointer to the PortBase servant regsitered
     * by registerPort().
     *
     * @param port_name The name of Port to be returned the servant pointer.
     * @return PortBase* Port's servant's pointer.
     *
     * @endif
     */
    PortBase* getPort(const char* port_name) const;


    /*!
     * @if jp
     *
     * @brief Port o^
     *
     *  port Ŏw肳ꂽ Port ̃T[ogo^B
     * o^ꂽ Port ̃T[og̓RXgN^ŗ^ꂽPOA 
     * activate ÃIuWFNgQƂPortProfileɃZbgB
     *
     * @param port Port T[og
     *
     * @else
     *
     * @brief Regsiter Port
     *
     * This operation registers the Port's servant given by argument.
     * The given Port's servant will be activated on the POA that is given
     * to the constructor, and the created object reference is set
     * to the Port's profile.
     *
     * @param port The Port's servant.
     *
     * @endif
     */
    void registerPort(PortBase& port);


    /*!
     * @if jp
     *
     * @brief Port ̓o^폜
     *
     *  port Ŏw肳ꂽ Port ̓o^폜B
     * 폜 Port  deactivate APortProfilẽt@Xɂ́A
     * nillB
     *
     * @param port Port T[og
     *
     * @else
     *
     * @brief Delete the Port's registration
     *
     * This operation unregisters the Port's registration.
     * When the Port is unregistered, Port is deactivated, and the object
     * reference in the Port's profile is set to nil.
     *
     * @param port The Port's servant.
     *
     * @endif
     */
    void deletePort(PortBase& port);

    /*!
     * @if jp
     *
     * @brief Port ̓o^폜
     *
     * Ŏw肳ꂽO Port ̓o^폜B
     * 폜 Port  deactivate APortProfilẽt@Xɂ́A
     * nillB
     *
     * @param port_name Port ̖O
     *
     * @else
     *
     * @brief Delete the Port' registration
     *
     * This operation delete the Port's registration specified by port_ name.
     * When the Port is unregistered, Port is deactivated, and the object
     * reference in the Port's profile is set to nil.
     *
     * @param port_name The Port's name.
     *
     * @endif
     */
    void deletePortByName(const char* port_name);


    /*!
     * @if jp
     *
     * @brief SĂ Port deactivateo^폜
     *
     * o^ĂSĂPortɑ΂āAT[ogdeactivatesA
     * o^Xg폜B
     *
     * @else
     *
     * @brief Unregister the Port
     *
     * This operation deactivates the all Port and deletes the all Port's
     * registrations from the list.
     *
     * @endif
     */
    void finalizePorts();

  private:
    // ORB ւ̃|C^
    CORBA::ORB_var m_pORB;

    // POA ւ̃|C^
    PortableServer::POA_var m_pPOA;

    // Port̃IuWFNgt@X̃Xg. PortList
    PortList m_portRefs;


    template <class T>
    class comp_op
    {
    public:
      comp_op(const char* _name)
	:
	m_name(_name)
      {
      }
      comp_op(T* obj) 
	: m_name((const char*)(obj->getProfile().name))
      {
      }
      bool operator()(T* obj)
      {
	std::string name((const char*)obj->getProfile().name);
	return m_name == name;
      }
    private:
      std::string m_name;
    };

    class port_prof_collect
    {
    public:
      port_prof_collect(PortProfileList& p) : m_p(p) {}
      void operator()(const PortBase* port)
      {
	CORBA_SeqUtil::push_back(m_p, port->getPortProfile());
      }
    private:
      PortProfileList& m_p;
    };
    
    struct find_port_name;
    struct del_port;

    // T[og𒼐ڊi[IuWFNg}l[W
    ObjectManager<const char*, PortBase, comp_op<PortBase> > m_portServants;


  };
};     // namespace RTC
#endif // PortAdmin_h
